@@ -4,7 +4,10 @@ import datetime
 from django_future.models import ScheduledJob
 
 
-def schedule_job(date, callable_name, content_object=None, expires=None,
+__all__ = ['schedule_job', 'job_as_parameter']
+
+
+def schedule_job(date, callable_name, content_object=None, expires='7d',
                  args=(), kwargs={}):
     """Schedule a job.
 
@@ -20,10 +23,14 @@ def schedule_job(date, callable_name, content_object=None, expires=None,
 
     """
     assert callable_name and isinstance(callable_name, str), callable_name
+    if isinstance(date, basestring):
+        date = _parse_timedelta(date)
     if isinstance(date, datetime.timedelta):
         date = datetime.datetime.now() + date
     job = ScheduledJob(callable_name=callable_name, time_slot_start=date)
     if expires:
+        if isinstance(expires, basestring):
+            expires = _parse_timedelta(expires)
         if isinstance(expires, datetime.timedelta):
             expires = date + expires
         job.time_slot_end = expires
@@ -32,18 +39,43 @@ def schedule_job(date, callable_name, content_object=None, expires=None,
     job.args = args
     job.kwargs = kwargs
     job.save()
+    return job
 
 
-def run_jobs():
-    """Run scheduled jobs."""
+_TIMEDELTA_SUFFIXES = {'m': 'minutes',
+                       'h': 'hours',
+                       'd': 'days',
+                       'w': 'weeks'}
+
+def _parse_timedelta(s):
+    n, suffix = int(s[:-1]), s[-1]
+    key = _TIMEDELTA_SUFFIXES[suffix]
+    kwargs = {key: n}
+    return datetime.timedelta(**kwargs)
+
+
+def job_as_parameter(f):
+    f.job_as_parameter = True
+    return f
+
+
+def run_jobs(delete_completed=False, now=None):
+    """Run scheduled jobs.
+
+    You may specify a date to be treated as the current time.
+    """
     # TODO: locking; make sure that several instances of ``run_jobs`` are not
     # running concurrently.
-    now = datetime.datetime.now()
+    if now is None:
+        now = datetime.datetime.now()
     jobs = ScheduledJob.objects.filter(status='scheduled',
                                        time_slot_start__lte=now,
                                        time_slot_end__gt=now)
     for job in jobs:
         # TODO: transactions
+        job.status = 'running'
+        job.execution_start = datetime.datetime.now()
+        job.save()
         try:
             job.run()
         except Exception:
@@ -53,5 +85,8 @@ def run_jobs():
             job.save()
             raise
         else:
-            job.status = 'complete'
-            job.save()
+            if delete_completed:
+                job.delete()
+            else:
+                job.status = 'complete'
+                job.save()
